@@ -1,12 +1,17 @@
-from .geometry import node_theta, get_cartesian, circos_radius
-from .utils import (infer_data_type, num_discrete_groups, cmaps,
-                    is_data_diverging)
-from matplotlib.path import Path
-from matplotlib.cm import get_cmap
+
+import logging
 
 import matplotlib.patches as patches
 import matplotlib.pyplot as plt
 import networkx as nx
+from matplotlib.cm import get_cmap
+from matplotlib.path import Path
+
+from .geometry import circos_radius, get_cartesian, node_theta
+from .utils import (cmaps, infer_data_type, is_data_diverging,
+                    num_discrete_groups)
+
+logging.basicConfig(level=logging.INFO)
 
 
 def despine(ax):
@@ -26,17 +31,50 @@ class BasePlot(object):
     keyword arguments specifying how nodes and edges should be styled and
     ordered.
 
-    An optional data_types dictionary can be passed in to bypass data type
+    An optional `data_types` dictionary can be passed in to bypass data type
     inference.
-    """
+
+    :param graph: A NetworkX graph object.
+    :type graph: `nx.Graph`, `nx.DiGraph`, `nx.MultiGraph`, `nx.MultiDiGraph`
+
+    :param node_order: The node attribute on which to specify the coloring of nodes.
+    :type node_order: `dict_key` (often `str`)
+
+    :param node_size: The node attribute on which to specify the size of nodes.
+    :type node_size: `dict_key` (often `str`)
+
+    :param node_grouping: The node attribute on which to specify the grouping position of nodes.
+    :type node_grouping: `dict_key` (often `str`)
+
+    :param node_color: The node attribute on which to specify the colour of nodes.
+    :type node_color: `dict_key` (often `str`)
+
+    :param node_labels: Boolean, whether to use node objects as labels or not.
+    :type node_labels: `bool`
+
+    :param edge_width: The edge attribute on which to specify the width of edges.
+    :type edge_with: `dict_key` (often `str`)
+
+    :param edge_color: The edge attribute on which to specify the colour of edges.
+    :type edge_color: `dict_key` (often `str`)
+
+    :param data_types: A mapping of node and edge data types that are stored.
+    :type data_types: `dict`
+
+    :param nodeprops: A `matplotlib`-compatible `props` dictionary.
+    :type nodeprops: `dict`
+
+    :param edgeprops: A `matplotlib-compatible `props` dictioanry.
+    :type edgeprops: `dict`
+    """  # noqa
     def __init__(self, graph, node_order=None, node_size=None,
-                 node_grouping=None, node_color=None, edge_width=None,
-                 edge_color=None, data_types=None, nodeprops=None,
-                 edgeprops=None):
+                 node_grouping=None, node_color=None, node_labels=None,
+                 edge_width=None, edge_color=None, data_types=None,
+                 nodeprops=None, edgeprops=None, **kwargs):
         super(BasePlot, self).__init__()
         # Set graph object
         self.graph = graph
-        self.nodes = graph.nodes()  # keep track of nodes separately.
+        self.nodes = list(graph.nodes())  # keep track of nodes separately.
 
         # Set node arrangement
         self.node_order = node_order
@@ -48,11 +86,14 @@ class BasePlot(object):
 
         # Set node colors
         self.node_color = node_color
+        self.sm = None  # sm -> for scalarmappable. See https://stackoverflow.com/questions/8342549/matplotlib-add-colorbar-to-a-sequence-of-line-plots  # noqa
+        logging.debug('INIT: {0}'.format(self.sm))
         if self.node_color:
             self.node_colors = []
             self.compute_node_colors()
         else:
             self.node_colors = ['blue'] * len(self.nodes)
+        self.node_labels = node_labels
 
         # Set edge properties
         self.edge_width = edge_width
@@ -65,7 +106,10 @@ class BasePlot(object):
             self.check_data_types(data_types)
             self.data_types = data_types
 
-        self.figure = plt.figure(figsize=(6, 6))
+        figsize = (6, 6)
+        if 'figsize' in kwargs.keys():
+            figsize = kwargs['figsize']
+        self.figure = plt.figure(figsize=figsize)
         self.ax = self.figure.add_subplot(1, 1, 1)
         despine(self.ax)
 
@@ -86,26 +130,43 @@ class BasePlot(object):
         # Compute each node's positions.
         self.compute_node_positions()
 
+        # Conditionally compute node label positions.
+        self.compute_node_label_positions()
+
     def check_data_types(self, data_types):
         """
         Checks the data_types passed into the Plot constructor and makes sure
-        that:
-        - the values passed in belong to 'ordinal', 'categorical', or
-          'continuous'.
+        that the values passed in belong to 'ordinal', 'categorical', or
+        'continuous'.
+
+        :param data_types: A dictionary mapping of data types.
+        :type data_types: `dict`
         """
         for k, v in data_types.items():
             assert v in ['ordinal', 'categorical', 'continuous']
 
     def draw(self):
+        """
+        Draws the Plot to screen.
+
+        If there is a continuous datatype for the nodes, it will be reflected
+        in self.sm being constructed (in `compute_node_colors`). It will then
+        automatically add in a colorbar to the plot and scale the plot axes
+        accordingly.
+        """
         self.draw_nodes()
         self.draw_edges()
+        logging.debug('DRAW: {0}'.format(self.sm))
+        if self.sm:
+            self.figure.subplots_adjust(right=0.8)
+            cax = self.figure.add_axes([0.85, 0.2, 0.05, 0.6])
+            self.figure.colorbar(self.sm, cax=cax)
         self.ax.relim()
         self.ax.autoscale_view()
+        self.ax.set_aspect('equal')
 
     def compute_node_colors(self):
-        """
-        Computes the node colors.
-        """
+        """Compute the node colors. Also computes the colorbar."""
         data = [self.graph.node[n][self.node_color] for n in self.nodes]
         data_reduced = sorted(list(set(data)))
         dtype = infer_data_type(data)
@@ -122,6 +183,17 @@ class BasePlot(object):
             idx = data_reduced.index(d) / n_grps
             self.node_colors.append(cmap(idx))
 
+        # Add colorbar if required.
+        logging.debug('length of data_reduced: {0}'.format(len(data_reduced)))
+        logging.debug('dtype: {0}'.format(dtype))
+        if len(data_reduced) > 1 and dtype == 'continuous':
+            self.sm = plt.cm.ScalarMappable(cmap=cmap,
+                                            norm=plt.Normalize(vmin=min(data_reduced),  # noqa
+                                                               vmax=max(data_reduced)   # noqa
+                                                               )
+                                            )
+            self.sm._A = []
+
     def compute_node_positions(self):
         """
         Computes the positions of each node on the plot.
@@ -130,6 +202,16 @@ class BasePlot(object):
         """
         pass
 
+    def compute_node_label_positions(self):
+        """
+        Computes the positions of each node's labels on the plot. The
+        horizontal and vertical alignment of the text varies according to the
+        location.
+
+        Needs to be implemented for each plot type.
+        """
+        pass
+
     def draw_nodes(self):
         """
         Renders the nodes to the plot or screen.
@@ -172,17 +254,6 @@ class CircosPlot(BasePlot):
     """
     Plotting object for CircosPlot.
     """
-    def __init__(self, graph, node_order=None, node_size=None,
-                 node_grouping=None, node_color=None, edge_width=None,
-                 edge_color=None, data_types=None, nodeprops=None,
-                 edgeprops=None):
-
-        # Initialize using BasePlot
-        BasePlot.__init__(self, graph, node_order=node_order,
-                          node_size=node_size, node_grouping=node_grouping,
-                          node_color=node_color, edge_width=edge_width,
-                          edge_color=edge_color, data_types=data_types,
-                          nodeprops=nodeprops, edgeprops=edgeprops)
 
     def compute_node_positions(self):
         """
@@ -201,6 +272,44 @@ class CircosPlot(BasePlot):
             ys.append(y)
         self.node_coords = {'x': xs, 'y': ys}
 
+    def compute_node_label_positions(self):
+        """
+        Uses the get_cartesian function to compute the positions of each node
+        label in the Circos plot.
+
+        This method is always called after the compute_node_positions
+        method, so that the plot_radius is pre-computed.
+        """
+        xs = []
+        ys = []
+        has = []
+        vas = []
+        for node in self.nodes:
+            theta = node_theta(self.nodes, node)
+            radius = self.plot_radius + self.nodeprops['radius']
+
+            x, y = get_cartesian(r=radius, theta=theta)
+
+            # Computes the text alignment
+            if x == 0:
+                ha = 'center'
+            elif x > 0:
+                ha = 'left'
+            else:
+                ha = 'right'
+            if y == 0:
+                va = 'middle'
+            elif y > 0:
+                va = 'bottom'
+            else:
+                va = 'top'
+            xs.append(x)
+            ys.append(y)
+            has.append(ha)
+            vas.append(va)
+        self.node_label_coords = {'x': xs, 'y': ys}  # node label coordinates
+        self.node_label_aligns = {'has': has, 'vas': vas}  # node label alignments  # noqa
+
     def draw_nodes(self):
         """
         Renders nodes to the figure.
@@ -215,6 +324,14 @@ class CircosPlot(BasePlot):
                                         lw=lw, color=color,
                                         zorder=2)
             self.ax.add_patch(node_patch)
+            if self.node_labels:
+                label_x = self.node_label_coords['x'][i]
+                label_y = self.node_label_coords['y'][i]
+                label_ha = self.node_label_aligns['has'][i]
+                label_va = self.node_label_aligns['vas'][i]
+                self.ax.text(s=node,
+                             x=label_x, y=label_y,
+                             ha=label_ha, va=label_va)
 
     def draw_edges(self):
         """
@@ -400,6 +517,10 @@ class MatrixPlot(BasePlot):
     def draw(self):
         """
         Draws the plot to screen.
+
+        Note to self: Do NOT call super(MatrixPlot, self).draw(); the
+        underlying logic for drawing here is completely different from other
+        plots, and as such necessitates a different implementation.
         """
         matrix = nx.to_numpy_matrix(self.graph, nodelist=self.nodes)
         self.ax.matshow(matrix, cmap=self.cmap)
@@ -409,17 +530,6 @@ class ArcPlot(BasePlot):
     """
     Plotting object for ArcPlot.
     """
-    def __init__(self, graph, node_order=None, node_size=None,
-                 node_grouping=None, node_color=None, edge_width=None,
-                 edge_color=None, data_types=None, nodeprops=None,
-                 edgeprops=None):
-
-        # Initialize using BasePlot
-        BasePlot.__init__(self, graph, node_order=node_order,
-                          node_size=node_size, node_grouping=node_grouping,
-                          node_color=node_color, edge_width=edge_width,
-                          edge_color=edge_color, data_types=data_types,
-                          nodeprops=nodeprops, edgeprops=edgeprops)
 
     def compute_node_positions(self):
         """
@@ -481,10 +591,7 @@ class ArcPlot(BasePlot):
             self.ax.add_patch(patch)
 
     def draw(self):
-        self.draw_nodes()
-        self.draw_edges()
+        super(ArcPlot, self).draw()
         xlimits = (-1, len(self.nodes) + 1)
-        # halfwidth = len(self.nodes) + 1 / 2
-        # ylimits = (-halfwidth, halfwidth)
         self.ax.set_xlim(*xlimits)
         self.ax.set_ylim(*xlimits)
